package parsec3

import channel.Adapter
import channel.CommandHost
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.launch
import kotlinx.coroutines.test.runTest
import kotlin.test.Test
import kotlin.test.assertEquals

internal class AdapterTest {

    class Api1: CommandHost<Unit>() {
        // create command `foo` that takes a string argument and
        // returns a string:
        val foo by command<String,String>()
    }

    class Api2: CommandHost<Unit>() {

        val bar by command<String,String>()
    }

    @Test
    fun interconnect() = runTest {
        val ch12 = Channel<ByteArray>()
        val ch21 = Channel<ByteArray>()
        val api1 = Api1()
        val api2 = Api2()
        api1.on(api1.foo) {
            it + "foo"
        }
        api2.on(api2.bar) {
            it + "bar"
        }

        val a1 = Adapter(Unit,api1) { ch12.send(it) }
        val a2 = Adapter(Unit,api2) { ch21.send(it) }
        launch { for( b in ch12) a2.receiveFrame(b) }
        launch { for( b in ch21) a1.receiveFrame(b) }

        assertEquals("123bar", a1.invokeCommand(api2.bar, "123"))
        assertEquals("321foo", a2.invokeCommand(api1.foo, "321"))

        ch12.cancel()
        ch21.cancel()

    }
}